% multinomial probit model
% using LDL' decomposition
% real data example 2 in the paper

clear;
nloop = 35000;
burnin = 5000;
load xtravel6.txt;
load ytravel6.txt;

K = 11; % dimension of beta
T = 5; % dimension of the covariance matrix
N = 522;
NT = N*T;

Y = zeros(N,1);
X = zeros(1*N,K);
xcount = 0;

% construt Y
for i = 1:N
    Y(i,1) = find( ytravel6(i, : ) ==1 ) - 1;
    X( (i-1)*T + 1: i*T, : ) = xtravel6( xcount + 2: xcount + T + 1, 1:K);
    xcount = xcount + T + 1;
end
Yhat = zeros(N*T,1);

% set parameters values
beta = zeros(T,1);;
Sig = eye(T);
invSig = inv( Sig );
L = eye(T);
D = eye(T);
Lc = L;

% initialize
sbeta = zeros( nloop - burnin, K ); 
sSigma = zeros( nloop - burnin, T+T-1);
slam = zeros( nloop - burnin, T );
Ytemp2 = zeros( N, 1 );
Ytemp2c = zeros( N, 1 );
sYhat = zeros( T * N, 1 );
Ymu = zeros( T * N, 1 );
sigj = zeros( 1, T - 1 ); % the jth row of Sigma minus the jth element
invSigjj = zeros( T - 1, T - 1 ); % T-1 by T-1 minor of Sigma by deleting the jth row and column,
sig = zeros( T * (T - 1) / 2, 1 );
sigc = zeros( T * (T - 1) / 2, 1 );
count = 0;

% prior
beta0 = zeros( K, 1 );
invVbeta = eye( K ) / 10;
Vbeta = inv( invVbeta);

nu = 50;
nu0 = nu + (1:T)'- T  ;
S0 = ones(T,1) * nu;
v0 = 1/nu; 

newnu = N + nu0;

disp('Starting MCMC.... ');
disp(' ' );
start_time = clock;

% Estimation

for loop = 1 : nloop

    % sample beta
    sumtemp = X' * kron( speye( N ), invSig );
    Db = inv( invVbeta + sumtemp * X );
    db = invVbeta * beta0 + sumtemp * Yhat;
    beta = Db*db + chol(Db)'*randn(K,1);
    
    % sample Yhat
    Ymu = X*beta;
   
    for j = 1 : T
        sigj = Sig( j, [ 1 : j - 1 j + 1 : end]);
        invSigjj = inv( Sig( [ 1 : j - 1 j + 1 : end] , [ 1 : j - 1 j + 1 : end]) );
        
        Ytemp1 = Yhat - Ymu;
        Ytemp1( j : T : end ) = 0;
        Yconmu = reshape( Ytemp1( find( Ytemp1 ~= 0 ) ), T - 1, N );  
        muj = Ymu( j : T : end ) + ( sigj * invSigjj * Yconmu)'; % compute the condistional mean
        vj = Sig(j, j) - sigj * invSigjj * sigj';    
        
        % compute the truncation region
        Yhat( j : T : end) = 0;
        max_yi = max( reshape( Yhat, T, N ) )';
        id1 = find( Y == j );
        n1 = length( id1 );
        id0 = find( Y ~= j );
        n0 = length( id0 );
        
        Ytemp2c( id0, 1 ) = tnormrnd( muj( id0, : ), vj * ones( n0, 1 ), -inf, max_yi( id0 ) );
        Ytemp2c( id1, 1 ) = tnormrnd( muj( id1, : ), vj * ones( n1, 1 ), max_yi( id1), inf );

        id = find(isinf(Ytemp2c)==1|isnan(Ytemp2c)==1); % search if there's any inf 
        Ytemp2c(id) = Ytemp2(id);
        Ytemp2 = Ytemp2c;
        Yhat( j : T : end ) = Ytemp2;
        
    end
      
    utemp = Yhat - Ymu;
    u = reshape( utemp, T, N)';

    % sample D
    w =  L * u';
    newS = diag( w * w' ) + S0;
    lambda =  1 ./ gamrnd( newnu / 2 , 2 ./ newS );
    D = diag( lambda );
    D(1,1) = 1;  
    
    % sample L in one block
    Dk = D(2,2)*inv( 1/v0 + u(:,1)' * u(:,1) );
    dk = - u(:,1)' * u(:,2) / D(2,2);
    Lc(2,1) = Dk*dk + chol(Dk)'*randn;

    lp = lmvnpdf(Lc(2,1),0,v0*D(2,2))-lmvnpdf(L(2,1),0,v0*D(2,2)) + lmvnpdf(L(2,1), Dk*dk, Dk ) - lmvnpdf( Lc(2,1),Dk*dk, Dk);
    for i = 3 : T
        Dk = D(i,i) * inv( 1/v0 + u(:, i-1 )' * u(:, i-1 ) );
        dk = - u(:, i-1 )' *( u(:,i) + u(:,1:i-2)*L(i,1:i-2)' ) / D(i,i);
        Lc(i,i-1) = Dk*dk + chol(Dk)'*randn;
        lp = lp + lmvnpdf( Lc(i,i-1),0,v0*D(i,i))-lmvnpdf(L(i,i-1),0,v0*D(i,i)) + lmvnpdf(L(i,i-1), Dk*dk, Dk ) - lmvnpdf( Lc(i,i-1),Dk*dk, Dk);
    end
    % restricting the matrix to be tridiagonal
    for i = 3:T
        for j = 1:i-2
            Lc(i,j) = Lc(i,i-1)*Lc(i-1,j);
        end
    end   
    
    w  = reshape(L * u', NT,1);
    wc = reshape(Lc * u',NT,1);
    llike = - w'*kron(speye(N),inv(sparse(D)))*w/2;
    llikec= -wc'*kron(speye(N),inv(sparse(D)))*wc/2;

    if (exp(llikec-llike+lp)> rand)
        L = Lc;
        if loop > burnin
            count = count + 1;
        end
    end

    invL = inv( L );
    Sig = invL * D * invL';
    invSig = inv(Sig);
    
    if loop > burnin
        sbeta( loop - burnin, : ) = beta';
        sSigma( loop - burnin, 1:T ) = diag(Sig);
        sSigma( loop - burnin, T+1:end ) = diag(Sig,1);
      end
       
    if ( mod( loop, 5000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end
    
end

disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

% compute p(\cdot>0)
betap = zeros(K,1);
sigp = zeros(9,1);

for loop=1:nloop-burnin
    Sig = sSigma(loop,:)';
    beta = sbeta(loop,:)';
    sigp = sigp + (Sig>0);
    betap = betap + (beta>0);
end
sigp = sigp/(nloop-burnin);
betap = betap/(nloop-burnin);


% Yhat = sYhat/(nloop - burnin);
Sighat = squeeze( mean( sSigma ) );
Sighatstd = squeeze( std( sSigma ) );
betahat = mean( sbeta )';
betastd = std( sbeta )';
